Package: SingleR
Title: Reference-Based Single-Cell RNA-Seq Annotation
Version: 2.14.1
Date: 2026-07-07
Authors@R: c(person("Dvir", "Aran", email="dvir.aran@ucsf.edu", role=c("aut", "cph")),
    person("Aaron", "Lun", email="infinite.monkeys.with.keyboards@gmail.com", role=c("ctb", "cre")),
    person("Daniel", "Bunis", role="ctb"),
    person("Jared", "Andrews", email = "jared.andrews07@gmail.com", role="ctb"),
    person("Friederike", "Dündar", email = "frd2007@med.cornell.edu", role="ctb"))
Description: Performs unbiased cell type recognition from single-cell RNA
    sequencing data, by leveraging reference transcriptomic datasets of pure cell
    types to infer the cell of origin of each single cell independently. 
License: GPL-3
Depends: SummarizedExperiment
Imports: methods, Matrix, BiocGenerics, S4Vectors, DelayedArray, stats,
        utils, Rcpp, beachmat (>= 2.27.3)
LinkingTo: Rcpp, beachmat, assorthead (>= 1.3.5)
Suggests: testthat, knitr, rmarkdown, BiocStyle, BiocParallel,
        SingleCellExperiment, scrapper (>= 1.5.16), scRNAseq, ggplot2,
        pheatmap, grDevices, gridExtra, viridis, celldex
biocViews: Software, SingleCell, GeneExpression, Transcriptomics,
        Classification, Clustering, Annotation
SystemRequirements: C++17
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.3
URL: https://github.com/SingleR-inc/SingleR
BugReports: https://github.com/SingleR-inc/SingleR/issues
git_url: https://git.bioconductor.org/packages/SingleR
git_branch: RELEASE_3_23
git_last_commit: 6f566f9
git_last_commit_date: 2026-07-07
Repository: Bioconductor 3.23
Date/Publication: 2026-07-09
NeedsCompilation: yes
Packaged: 2026-07-10 00:22:47 UTC; biocbuild
Author: Dvir Aran [aut, cph],
  Aaron Lun [ctb, cre],
  Daniel Bunis [ctb],
  Jared Andrews [ctb],
  Friederike Dündar [ctb]
Maintainer: Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>
