# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

classify_integrated <- function(test, results, integrated_build, quantile, use_fine_tune, fine_tune_threshold, nthreads) {
    .Call('_SingleR_classify_integrated', PACKAGE = 'SingleR', test, results, integrated_build, quantile, use_fine_tune, fine_tune_threshold, nthreads)
}

#' @importFrom Rcpp sourceCpp
#' @useDynLib SingleR
classify_single <- function(test, prebuilt, quantile, use_fine_tune, fine_tune_threshold, nthreads) {
    .Call('_SingleR_classify_single', PACKAGE = 'SingleR', test, prebuilt, quantile, use_fine_tune, fine_tune_threshold, nthreads)
}

find_classic_markers <- function(nlabels, ngenes, labels, ref, de_n, nthreads) {
    .Call('_SingleR_find_classic_markers', PACKAGE = 'SingleR', nlabels, ngenes, labels, ref, de_n, nthreads)
}

grouped_medians <- function(ref, groups, ngroups, nthreads) {
    .Call('_SingleR_grouped_medians', PACKAGE = 'SingleR', ref, groups, ngroups, nthreads)
}

train_integrated <- function(test_features, references, ref_features, labels, prebuilt, nthreads) {
    .Call('_SingleR_train_integrated', PACKAGE = 'SingleR', test_features, references, ref_features, labels, prebuilt, nthreads)
}

#' @importFrom Rcpp sourceCpp
#' @useDynLib SingleR
train_single <- function(test_features, ref, ref_features, labels, markers, builder, nthreads) {
    .Call('_SingleR_train_single', PACKAGE = 'SingleR', test_features, ref, ref_features, labels, markers, builder, nthreads)
}

get_ref_subset <- function(built) {
    .Call('_SingleR_get_ref_subset', PACKAGE = 'SingleR', built)
}

is_valid_built <- function(built) {
    .Call('_SingleR_is_valid_built', PACKAGE = 'SingleR', built)
}

